@@ -45,8 +45,10 @@
 #include <dbus/dbus.h>
 #include <gdbus.h>
 
-#include "hcid.h"
+#include "logging.h"
 #include "textfile.h"
+
+#include "hcid.h"
 #include "manager.h"
 #include "adapter.h"
 #include "device.h"
@@ -54,7 +56,6 @@
 #include "glib-helper.h"
 #include "dbus-common.h"
 #include "agent.h"
-#include "dbus-hci.h"
 
 static DBusConnection *connection = NULL;
 
@@ -119,7 +120,7 @@ int dev_rssi_cmp(struct remote_dev_info *d1, struct remote_dev_info *d2)
 	return rssi1 - rssi2;
 }
 
-int found_device_add(GSList **list, bdaddr_t *bdaddr, int8_t rssi,
+static int found_device_add(GSList **list, bdaddr_t *bdaddr, int8_t rssi,
 			name_status_t name_status)
 {
 	struct remote_dev_info *dev, match;
@@ -271,21 +272,21 @@ static void adapter_mode_changed(struct adapter *adapter, uint8_t scan_mode)
 	switch (scan_mode) {
 	case SCAN_DISABLED:
 		mode = "off";
-		adapter->mode = MODE_OFF;
+		adapter_set_mode(adapter, MODE_OFF);
 		break;
 	case SCAN_PAGE:
 		mode = "connectable";
-		adapter->mode = MODE_CONNECTABLE;
+		adapter_set_mode(adapter, MODE_CONNECTABLE);
 		break;
 	case (SCAN_PAGE | SCAN_INQUIRY):
 
 		if (adapter->discov_timeout != 0)
 			adapter_set_discov_timeout(adapter, adapter->discov_timeout * 1000);
 
-		if (adapter->mode == MODE_LIMITED) {
+		if (adapter_get_mode(adapter) == MODE_LIMITED) {
 			mode = "limited";
 		} else {
-			adapter->mode = MODE_DISCOVERABLE;
+			adapter_set_mode(adapter, MODE_DISCOVERABLE);
 			mode = "discoverable";
 		}
 		break;
@@ -800,6 +801,8 @@ void hcid_dbus_inquiry_start(bdaddr_t *local)
 {
 	struct adapter *adapter;
 	const gchar *path;
+	gboolean discov_active;
+	int state;
 
 	adapter = manager_find_adapter(local);
 	if (!adapter) {
@@ -807,30 +810,36 @@ void hcid_dbus_inquiry_start(bdaddr_t *local)
 		return;
 	}
 
-	adapter->discov_active = 1;
+	state = adapter_get_state(adapter);
+	state |= STD_INQUIRY;
+	adapter_set_state(adapter, state);
 	/*
 	 * Cancel pending remote name request and clean the device list
 	 * when inquiry is supported in periodic inquiry idle state.
 	 */
-	if (adapter->pdiscov_active)
+	if (adapter_get_state(adapter) & PERIODIC_INQUIRY)
 		pending_remote_name_cancel(adapter);
 
 	/* Disable name resolution for non D-Bus clients */
-	if (!adapter->discov_requestor)
-		adapter->discov_type &= ~RESOLVE_NAME;
-
+	if (!adapter->discov_requestor) {
+		state = adapter_get_state(adapter);
+		state &= ~RESOLVE_NAME;
+		adapter_set_state(adapter, state);
+	}
 	path = adapter_get_path(adapter);
 
+	discov_active = (adapter_get_state(adapter) & STD_INQUIRY) ? TRUE : FALSE;
+
 	dbus_connection_emit_property_changed(connection, path,
 			ADAPTER_INTERFACE, "PeriodicDiscovery",
-			DBUS_TYPE_BOOLEAN, &adapter->discov_active);
+			DBUS_TYPE_BOOLEAN, &discov_active);
 
 	g_dbus_emit_signal(connection, path,
 			ADAPTER_INTERFACE, "DiscoveryStarted",
 			DBUS_TYPE_INVALID);
 }
 
-int found_device_req_name(struct adapter *adapter)
+static int found_device_req_name(struct adapter *adapter)
 {
 	struct hci_request rq;
 	evt_cmd_status rp;
@@ -929,6 +938,7 @@ void hcid_dbus_inquiry_complete(bdaddr_t *local)
 	struct remote_dev_info *dev;
 	bdaddr_t tmp;
 	const gchar *path;
+	int state;
 
 	adapter = manager_find_adapter(local);
 	if (!adapter) {
@@ -939,7 +949,8 @@ void hcid_dbus_inquiry_complete(bdaddr_t *local)
 	path = adapter_get_path(adapter);
 
 	/* Out of range verification */
-	if (adapter->pdiscov_active && !adapter->discov_active) {
+	if ((adapter_get_state(adapter) & PERIODIC_INQUIRY) &&
+				!(adapter_get_state(adapter) & STD_INQUIRY)) {
 		GSList *l;
 
 		send_out_of_range(path, adapter->oor_devices);
@@ -964,9 +975,11 @@ void hcid_dbus_inquiry_complete(bdaddr_t *local)
 	 * Enable resolution again: standard inquiry can be
 	 * received in the periodic inquiry idle state.
 	 */
-	if (adapter->pdiscov_requestor && adapter->pdiscov_resolve_names)
-		adapter->discov_type |= RESOLVE_NAME;
-
+	if (adapter->pdiscov_requestor) {
+		state = adapter_get_state(adapter);
+		state |= RESOLVE_NAME;
+		adapter_set_state(adapter, state);
+	}
 	/*
 	 * The following scenarios can happen:
 	 * 1. standard inquiry: always send discovery completed signal
@@ -982,12 +995,14 @@ void hcid_dbus_inquiry_complete(bdaddr_t *local)
 	if (!found_device_req_name(adapter))
 		return;		/* skip - there is name to resolve */
 
-	if (adapter->discov_active) {
+	if (adapter_get_state(adapter) & STD_INQUIRY) {
 		g_dbus_emit_signal(connection, path,
 				ADAPTER_INTERFACE, "DiscoveryCompleted",
 				DBUS_TYPE_INVALID);
 
-		adapter->discov_active = 0;
+		state = adapter_get_state(adapter);
+		state &= ~STD_INQUIRY;
+		adapter_set_state(adapter, state);
 	}
 
 	/* free discovered devices list */
@@ -1012,7 +1027,9 @@ void hcid_dbus_inquiry_complete(bdaddr_t *local)
 		}
 
 		/* reset the discover type for standard inquiry only */
-		adapter->discov_type &= ~STD_INQUIRY;
+		state = adapter_get_state(adapter);
+		state &= ~STD_INQUIRY;
+		adapter_set_state(adapter, state);
 	}
 }
 
@@ -1020,6 +1037,8 @@ void hcid_dbus_periodic_inquiry_start(bdaddr_t *local, uint8_t status)
 {
 	struct adapter *adapter;
 	const gchar *path;
+	gboolean discov_active;
+	int state;
 
 	/* Don't send the signal if the cmd failed */
 	if (status)
@@ -1031,23 +1050,32 @@ void hcid_dbus_periodic_inquiry_start(bdaddr_t *local, uint8_t status)
 		return;
 	}
 
-	adapter->pdiscov_active = 1;
+	state = adapter_get_state(adapter);
+	state |= PERIODIC_INQUIRY;
+	adapter_set_state(adapter, state);
 
 	/* Disable name resolution for non D-Bus clients */
-	if (!adapter->pdiscov_requestor)
-		adapter->discov_type &= ~RESOLVE_NAME;
+	if (!adapter->pdiscov_requestor) {
+		state = adapter_get_state(adapter);
+		state &= ~RESOLVE_NAME;
+		adapter_set_state(adapter, state);
+	}
 
 	path = adapter_get_path(adapter);
 
+	discov_active = (adapter_get_state(adapter) & PERIODIC_INQUIRY) ? TRUE:FALSE;
+
 	dbus_connection_emit_property_changed(connection, path,
 				ADAPTER_INTERFACE, "PeriodicDiscovery",
-				DBUS_TYPE_BOOLEAN, &adapter->pdiscov_active);
+				DBUS_TYPE_BOOLEAN, &discov_active);
 }
 
 void hcid_dbus_periodic_inquiry_exit(bdaddr_t *local, uint8_t status)
 {
 	struct adapter *adapter;
 	const gchar *path;
+	gboolean discov_active;
+	int state;
 
 	/* Don't send the signal if the cmd failed */
 	if (status)
@@ -1061,8 +1089,9 @@ void hcid_dbus_periodic_inquiry_exit(bdaddr_t *local, uint8_t status)
 
 	/* reset the discover type to be able to handle D-Bus and non D-Bus
 	 * requests */
-	adapter->pdiscov_active = 0;
-	adapter->discov_type &= ~(PERIODIC_INQUIRY | RESOLVE_NAME);
+	state = adapter_get_state(adapter);
+	state &= ~(PERIODIC_INQUIRY | RESOLVE_NAME);
+	adapter_set_state(adapter, state);
 
 	/* free discovered devices list */
 	g_slist_foreach(adapter->found_devices, (GFunc) g_free, NULL);
@@ -1085,18 +1114,22 @@ void hcid_dbus_periodic_inquiry_exit(bdaddr_t *local, uint8_t status)
 
 	/* workaround: inquiry completed is not sent when exiting from
 	  * periodic inquiry */
-	if (adapter->discov_active) {
+	if (adapter_get_state(adapter) & STD_INQUIRY) {
 		g_dbus_emit_signal(connection, path,
 				ADAPTER_INTERFACE, "DiscoveryCompleted",
 				DBUS_TYPE_INVALID);
 
-		adapter->discov_active = 0;
+		state = adapter_get_state(adapter);
+		state &= ~STD_INQUIRY;
+		adapter_set_state(adapter, state);
 	}
 
+	discov_active = (adapter_get_state(adapter) & STD_INQUIRY) ? TRUE : FALSE;
+
 	/* Send discovery completed signal if there isn't name to resolve */
 	dbus_connection_emit_property_changed(connection, path,
 				ADAPTER_INTERFACE, "PeriodicDiscovery",
-				DBUS_TYPE_BOOLEAN, &adapter->discov_active);
+				DBUS_TYPE_BOOLEAN, &discov_active);
 }
 
 static char *extract_eir_name(uint8_t *data, uint8_t *type)
@@ -1182,6 +1215,7 @@ void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 	uint8_t name_type = 0x00;
 	name_status_t name_status;
 	const gchar *path;
+	int state;
 
 	ba2str(local, local_addr);
 	ba2str(peer, peer_addr);
@@ -1201,11 +1235,15 @@ void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 	 * workaround to identify situation when the daemon started and
 	 * a standard inquiry or periodic inquiry was already running
 	 */
-	if (!adapter->discov_active && !adapter->pdiscov_active)
-		adapter->pdiscov_active = 1;
+	if (!(adapter_get_state(adapter) & STD_INQUIRY) &&
+			!(adapter_get_state(adapter) & PERIODIC_INQUIRY)) {
+		state = adapter_get_state(adapter);
+		state |= PERIODIC_INQUIRY;
+		adapter_set_state(adapter, state);
+	}
 
 	/* reset the idle flag when the inquiry complete event arrives */
-	if (adapter->pdiscov_active) {
+	if (adapter_get_state(adapter) & PERIODIC_INQUIRY) {
 		adapter->pinq_idle = 0;
 
 		/* Out of range list update */
@@ -1229,7 +1267,7 @@ void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 		return;
 
 	/* the inquiry result can be triggered by NON D-Bus client */
-	if (adapter->discov_type & RESOLVE_NAME)
+	if (adapter_get_state(adapter) & RESOLVE_NAME)
 		name_status = NAME_REQUIRED;
 	else
 		name_status = NAME_NOT_REQUIRED;
@@ -1326,6 +1364,7 @@ void hcid_dbus_remote_name(bdaddr_t *local, bdaddr_t *peer, uint8_t status,
 	const char *paddr = peer_addr;
 	const gchar *dev_path;
 	const gchar *path;
+	int state;
 
 	adapter = manager_find_adapter(local);
 	if (!adapter) {
@@ -1380,18 +1419,21 @@ void hcid_dbus_remote_name(bdaddr_t *local, bdaddr_t *peer, uint8_t status,
 		}
 
 		/* Disable name resolution for non D-Bus clients */
-		if (!adapter->pdiscov_requestor)
-			adapter->discov_type &= ~RESOLVE_NAME;
+		if (!adapter->pdiscov_requestor) {
+			state = adapter_get_state(adapter);
+			state &= ~RESOLVE_NAME;
+			adapter_set_state(adapter, state);
+		}
 	}
-
 	path = adapter_get_path(adapter);
 
-	if (adapter->discov_active) {
+	if (adapter_get_state(adapter) & STD_INQUIRY) {
 		g_dbus_emit_signal(connection, path,
 				ADAPTER_INTERFACE, "DiscoveryCompleted",
 				DBUS_TYPE_INVALID);
-
-		adapter->discov_active = 0;
+		state = adapter_get_state(adapter);
+		state &= ~STD_INQUIRY;
+		adapter_set_state(adapter, state);
 	}
 }
 
@@ -1428,10 +1470,9 @@ void hcid_dbus_conn_complete(bdaddr_t *local, uint8_t status, uint16_t handle,
 
 		device = adapter_find_device(adapter, paddr);
 		if (device) {
+			dev_path = device_get_path(device);
 
-		  dev_path = device_get_path(device);
-
-		  dbus_connection_emit_property_changed(connection,
+			dbus_connection_emit_property_changed(connection,
 					dev_path, DEVICE_INTERFACE,
 					"Connected", DBUS_TYPE_BOOLEAN,
 					&connected);
@@ -1779,6 +1820,7 @@ int hcid_dbus_get_io_cap(bdaddr_t *local, bdaddr_t *remote,
 
 	ba2str(remote, addr);
 
+	/* For CreatePairedDevice use dedicated bonding */
 	device = adapter_find_device(adapter, addr);
 	if (device) {
 		agent = device_get_agent(device);
@@ -1789,15 +1831,31 @@ int hcid_dbus_get_io_cap(bdaddr_t *local, bdaddr_t *remote,
 		agent = adapter->agent;
 
 	if (!agent) {
-		if (!(*auth & 0x01)) {
+		/* This is the non bondable mode case */
+		if (device && device_get_auth(device) > 0x01) {
+			debug("Bonding request, but no agent present");
+			return -1;
+		}
+
+		/* No agent available, and no bonding case */
+		if (*auth < 0x02) {
+			debug("Allowing no bonding without agent");
 			/* No input, no output */
 			*cap = 0x03;
 			return 0;
 		}
+
 		error("No agent available for IO capability");
 		return -1;
 	}
 
+	if (device && *auth < 0x02) {
+		/* If remote requests dedicated bonding follow that lead */
+		if (device_get_auth(device) == 0x02 ||
+				device_get_auth(device) == 0x03)
+			*auth = 0x02;
+	}
+
 	*cap = agent_get_io_capability(agent);
 
 	return 0;
@@ -1886,8 +1944,9 @@ int cancel_discovery(struct adapter *adapter)
 	GSList *l;
 	int dd, err = 0;
 	uint16_t dev_id = adapter_get_dev_id(adapter);
+	int state;
 
-	if (!adapter->discov_active)
+	if (!(adapter_get_state(adapter) & STD_INQUIRY))
 		goto cleanup;
 
 	dd = hci_open_dev(dev_id);
@@ -1933,8 +1992,11 @@ cleanup:
 	adapter->found_devices = NULL;
 
 	/* Disable name resolution for non D-Bus clients */
-	if (!adapter->pdiscov_requestor)
-		adapter->discov_type &= ~RESOLVE_NAME;
+	if (!adapter->pdiscov_requestor) {
+		state = adapter_get_state(adapter);
+		state &= ~RESOLVE_NAME;
+		adapter_set_state(adapter, state);
+	}
 
 	return err;
 }
@@ -1969,7 +2031,7 @@ int cancel_periodic_discovery(struct adapter *adapter)
 	int dd, err = 0;
 	uint16_t dev_id = adapter_get_dev_id(adapter);
 
-	if (!adapter->pdiscov_active)
+	if (!(adapter_get_state(adapter) & PERIODIC_INQUIRY))
 		goto cleanup;
 
 	dd = hci_open_dev(dev_id);
